@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2023 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -23,6 +23,7 @@ distributions.  doctest discovery currently requires file system traversal.
 
 import os
 import sys
+import shlex
 
 from click.testing import CliRunner
 from contextlib import ExitStack
@@ -30,7 +31,8 @@ from importlib import import_module
 from mailman.testing.helpers import call_api
 from mailman.testing.layers import SMTPLayer
 from public import public
-from subprocess import PIPE, STDOUT, run
+from subprocess import PIPE, run, STDOUT
+from urllib.error import HTTPError
 
 
 DOT = '.'
@@ -93,7 +95,11 @@ def call_http(url, data=None, method=None, username=None, password=None):
     :return: The decoded JSON data structure.
     :raises HTTPError: when a non-2xx return code is received.
     """
-    content, response = call_api(url, data, method, username, password)
+    try:
+        content, response = call_api(url, data, method, username, password)
+    except HTTPError as ex:
+        print(ex)
+        return
     if content is None:
         # We used to use httplib2 here, which included the status code in the
         # response headers in the `status` key.  requests doesn't do this, but
@@ -119,7 +125,8 @@ def _print_dict(data, depth=0):
         print('    ' * depth + '{}: {}'.format(item, value))
 
 
-def dump_json(url, data=None, method=None, username=None, password=None):
+def dump_json(url, data=None, method=None, username=None, password=None,
+              sort_entries=None):
     """Print the JSON dictionary read from a URL.
 
     :param url: The url to open, read, and print.
@@ -133,7 +140,9 @@ def dump_json(url, data=None, method=None, username=None, password=None):
     :type username: str
     :param password: The HTTP Basic Auth password.  None means use the value
         from the configuration.
-    :type username: str
+    :type password: str
+    :param sort_entries: The key to sort entiries.
+    :type sort_entries: str
     """
     results = call_http(url, data, method, username, password)
     if results is None:
@@ -141,7 +150,10 @@ def dump_json(url, data=None, method=None, username=None, password=None):
     for key in sorted(results):
         value = results[key]
         if key == 'entries':
-            for i, entry in enumerate(value):
+            entries = value
+            if sort_entries:
+                entries = sorted(entries, key=lambda x: x[sort_entries])
+            for i, entry in enumerate(entries):
                 # entry is a dictionary.
                 print('entry %d:' % i)
                 for entry_key in sorted(entry):
@@ -169,7 +181,7 @@ def cli(command_path):
     package_path, dot, name = command_path.rpartition('.')
     command = getattr(import_module(package_path), name)
     def inner(command_string, quiet=False, input=None):           # noqa: E306
-        args = command_string.split()
+        args = shlex.split(command_string)
         assert args[0] == 'mailman', args
         assert args[1] == command.name, args
         # The first two will be `mailman <command>`.  That's just for
@@ -191,6 +203,10 @@ def run_mailman(args, **overrides):
     env = os.environ.copy()
     env.update(overrides)
     run_args = [exe]
+    # When running tests as root, just add the flag to force run mailman
+    # command without errors.
+    if os.geteuid() == 0:
+        run_args.append('--run-as-root')
     run_args.extend(args)
     proc = run(
         run_args, env=env, stdout=PIPE, stderr=STDOUT, universal_newlines=True)
